unit VaxArrayBuff;

interface

uses
  System.Classes, Windows,
  System.Threading, System.Diagnostics, System.SysUtils;


type TVaxArrayBuff = class(Tobject)

  private
  m_aMainBUFF : PByte;
  m_nTotalSize: Integer;

  m_nWriteIndex: Integer;
  m_nGrowSize: Integer;

  public
  constructor Create();
  destructor Destroy(); override;

  private
  procedure AllocBUFF();
  procedure AdjustBUFF(nDataSize: Integer);

  public
  procedure ResetBUFF();
  procedure SetSizeGrow(nSize: Integer);

  function GetDataBUFF() : PByte;
  function GetSizeBUFF() : Integer;

  procedure WriteToBUFF(aDataBuff: PByte; nSizeBuff: Integer);

end;

implementation

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  constructor TVaxArrayBuff.Create();
  begin

    m_aMainBUFF := nil;
    m_nTotalSize := 0;

	  m_nWriteIndex := 0;
	  m_nGrowSize := 16000;

  end;

  destructor TVaxArrayBuff.Destroy();
  begin

    ResetBUFF();

    Inherited;

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxArrayBuff.ResetBUFF();
  begin

    m_nWriteIndex := 0;

	  if(m_aMainBUFF <> nil) Then
	  begin
      FreeMem(m_aMainBUFF);
		  m_aMainBUFF := nil;
	  end;

	  m_nTotalSize := 0;

  end;

  procedure TVaxArrayBuff.AllocBUFF();
  begin

    if(m_aMainBUFF <> nil) Then
	  begin
      Exit();
	  end;

    GetMem(m_aMainBUFF, m_nGrowSize);
    m_nTotalSize := m_nGrowSize;

  end;

  procedure TVaxArrayBuff.SetSizeGrow(nSize: Integer);
  begin

    m_nGrowSize := nSize;

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxArrayBuff.AdjustBUFF(nDataSize: Integer);
  var
    nTotalSize: Integer;
    aDataBUFF: PByte;
  begin

    if ((m_nWriteIndex + nDataSize) <= m_nTotalSize) Then
      Exit();

    nTotalSize := m_nTotalSize;

    While true do
    begin

      nTotalSize := nTotalSize + m_nGrowSize;

		  if((m_nWriteIndex + nDataSize) <= nTotalSize) Then
			  break;

    end;

    GetMem(aDataBUFF, nTotalSize);

    CopyMemory(aDataBUFF, m_aMainBUFF, m_nWriteIndex);

    FreeMem(m_aMainBUFF);
    m_aMainBUFF := nil;

    m_aMainBUFF :=  aDataBUFF;
    m_nTotalSize := nTotalSize;

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxArrayBuff.WriteToBUFF(aDataBuff: PByte; nSizeBuff: Integer);
  var
    nWriteIndex: Integer;
  begin

    AllocBUFF();

    if(nSizeBuff <= 0) Then
		  Exit();

    AdjustBUFF(nSizeBuff);

	  nWriteIndex := m_nWriteIndex;
    m_nWriteIndex := m_nWriteIndex + nSizeBuff;

    CopyMemory(m_aMainBUFF + nWriteIndex, aDataBuff, nSizeBuff);

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  function TVaxArrayBuff.GetDataBUFF() : PByte;
  begin

  	Result := m_aMainBUFF;

  end;

  function TVaxArrayBuff.GetSizeBUFF() : Integer;
  begin

  	Result := m_nWriteIndex;

  end;

end.
