unit VaxPipeClient;

interface

uses
  System.Classes, Windows,
  System.SysUtils;


type TVaxPipeClient = class

  private
  m_hHandlePipe: THandle;
  m_hWaitEvent : THandle;

  m_objOverLap: Overlapped;
  m_aDataReadPipe : PByte;

  public
  constructor Create();
  destructor Destroy(); override;

  public
  function CreatePipe(sPipeName: String) : Boolean;
  procedure ClosePipe();

  function BeginReadPipe(var hWaitEvent: THandle) : Boolean;
  function EndReadPipe(var aDataRead: PByte; var nSizeRead: Integer) : Boolean;

end;

implementation

  const PIPE_BUFFER_LENGTH = 64000;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  constructor TVaxPipeClient.Create();
  begin

    m_hHandlePipe := 0;
    m_hWaitEvent := 0;

    GetMem(m_aDataReadPipe, PIPE_BUFFER_LENGTH);

  end;

  destructor TVaxPipeClient.Destroy();
  begin

    ClosePipe();

    if(m_aDataReadPipe <> nil) Then
    begin
      FreeMem(m_aDataReadPipe);
      m_aDataReadPipe := nil;
    end;

    Inherited;

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  function TVaxPipeClient.CreatePipe(sPipeName : String) : Boolean;
  var
    dwMode : DWORD;
  begin

    m_hHandlePipe := CreateFile(PChar(sPipeName), GENERIC_READ OR FILE_WRITE_ATTRIBUTES, 0, NIL, OPEN_EXISTING, FILE_FLAG_OVERLAPPED, 0);

    if (m_hHandlePipe = INVALID_HANDLE_VALUE) then
    begin

      dwMode := GetLastError();

      if (GetLastError() <> ERROR_PIPE_BUSY) then
      begin
        Exit(false);
      end;

      if not WaitNamedPipe(PChar(sPipeName), 3000) then
      begin
        Exit(false);
      end;

    end;

    //Result := (m_hHandlePipe <> INVALID_HANDLE_VALUE);

    dwMode := PIPE_READMODE_MESSAGE;

    SetNamedPipeHandleState(m_hHandlePipe, dwMode, NIL, NIL);

     m_hWaitEvent := CreateEvent(NIL, true, false, NIL);

     if(m_hWaitEvent = 0) Then
     begin

      ClosePipe();
      Exit(false);

     end;

    Exit(true);

  end;

  procedure TVaxPipeClient.ClosePipe();
  begin

    if(m_hHandlePipe <> 0) Then
    begin
      CloseHandle(m_hHandlePipe);
      m_hHandlePipe := 0;
    end;

    if(m_hWaitEvent <> 0) Then
    begin
      CloseHandle(m_hWaitEvent);
      m_hWaitEvent := 0;
    end;

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  function TVaxPipeClient.BeginReadPipe(var hWaitEvent: THandle) : Boolean;
  var
    bResult: Boolean;
    dwSizeRead: DWORD;
  begin

    FillChar(m_objOverLap, sizeof(m_objOverLap), #0);
    m_objOverLap.hEvent := m_hWaitEvent;

    bResult := ReadFile(m_hHandlePipe, m_aDataReadPipe^, PIPE_BUFFER_LENGTH, dwSizeRead, @m_objOverLap);

    if(bResult = true) Then
    begin
      Exit(true);
    end;

    if ((bResult <> false) OR (GetLastError() <> ERROR_IO_PENDING)) Then
     begin
      Exit(false);
    end;

    //((bResult = false) AND (GetLastError() = ERROR_MORE_DATA)) Then

    hWaitEvent := m_hWaitEvent;
    Exit(true);

  end;


  function TVaxPipeClient.EndReadPipe(var aDataRead: PByte; var nSizeRead: Integer) : Boolean;
  var
    dwSizeRead: DWORD;
  begin

    if not GetOverlappedResult(m_hHandlePipe, m_objOverLap, dwSizeRead, False) Then
      Exit(False);

    nSizeRead := dwSizeRead;
    aDataRead := m_aDataReadPipe;

    Exit(true);

  end;


  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

end.
