unit VaxCallSession;

interface

uses
  System.Classes,
  GenFileLocation,
  VaxDefine, VaxAccessAudioPCM,
  System.SysUtils;


type TVaxCallSession = class(TObject)

  private
  mAccessPCM: TVaxAccessAudioPCM;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  public
  procedure OnCallSessionCreated(nSessionId: Integer; nReasonCode: Integer);
  procedure OnCallSessionClosed(nSessionId: Integer; nReasonCode: Integer);

  procedure OnIncomingCall(nSessionId: Integer; sCallerName: String; sCallerId: String; sCalleeName: String; sCalleeId: String; sDialNo: String; nFromPeerType: Integer; sFromPeerName: String; sUserAgentName: String; sRecXML: String; sFromIP: String; nFromPort: Integer);
  procedure OnCallSessionConnected(nSessionId: Integer);

  procedure OnCallSessionAccessAudioPCM(nSessionId: Integer; nAccessType: Integer; pDataPCM: Pointer; nSizePCM: Integer);

  public
  constructor Create();
  destructor Destroy(); override;

  private
  function AcceptCallSession(nSessionId: Integer) : Boolean;
  procedure StartRecording(nSessionId: Integer);

end;

implementation

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  Uses VaxServerREC, VaxServerCOM;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  constructor TVaxCallSession.Create();
  begin

    inherited Create();

  end;

  destructor TVaxCallSession.Destroy();
  begin

    inherited;

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxCallSession.OnCallSessionCreated(nSessionId: Integer; nReasonCode: Integer);
  begin

    mAccessPCM := TVaxAccessAudioPCM.Create(nSessionId);

  end;

  procedure TVaxCallSession.OnCallSessionClosed(nSessionId: Integer; nReasonCode: Integer);
  begin

    mAccessPCM.Destroy();

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  function TVaxCallSession.AcceptCallSession(nSessionId: Integer) : Boolean;
  var
    bResult : Boolean;
  begin

    bResult := TVaxServerREC.mVaxREC.AcceptCallSession(nSessionId);

    if(bResult = False) then
    begin
      TVaxServerREC.mVaxREC.RejectCallSession(nSessionId, 404, 'Not Found');
      Exit(False);
    end;

    Result := True;

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxCallSession.StartRecording(nSessionId: Integer);
  var
    objGenFileLoc : TGenFileLocation;
    sFilePath : string;
    sErrorMsg : string;
  begin

    objGenFileLoc := TGenFileLocation.Create();

    sFilePath := objGenFileLoc.GenRecordingFilePath(nSessionId, TVaxServerREC.mVaxREC.GetRecordingFolder());

    if(sFilePath = '') Then
    begin

      sErrorMsg := Format('[SessionId: %d] [Error Msg: %s]', [nSessionId, TVaxDefine.VAX_ERROR_REC_PATH_NOT_FOUND]);
      TVaxServerREC.mVaxREC.GenStatusLog(sErrorMsg);

      Exit();
    end;

    TVaxServerREC.mVaxREC.RecordWaveStartToCallSession(nSessionId, sFilePath);

  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxCallSession.OnIncomingCall(nSessionId: Integer; sCallerName: String; sCallerId: String; sCalleeName: String; sCalleeId: String; sDialNo: String; nFromPeerType: Integer; sFromPeerName: String; sUserAgentName: String; sRecXML: String; sFromIP: String; nFromPort: Integer);
  begin
    AcceptCallSession(nSessionId);
  end;

  procedure TVaxCallSession.OnCallSessionConnected(nSessionId: Integer);
  begin
    StartRecording(nSessionId);
  end;

  //////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////

  procedure TVaxCallSession.OnCallSessionAccessAudioPCM(nSessionId: Integer; nAccessType: Integer; pDataPCM: Pointer; nSizePCM: Integer);
  begin
    mAccessPCM.OnCallSessionAccessAudioPCM(nSessionId, nAccessType, pDataPCM, nSizePCM);
  end;

end.

